/* $Id$ */

#ifndef IMAGE_IMAGE_EXCEPTION_HPP
#define IMAGE_IMAGE_EXCEPTION_HPP
#include <opencv/cxcore.h>
#if CV_MAJOR_VERSION*10 + CV_MINOR_VERSION > 21
#include <opencv2/core/core_c.h>
#else
#include <cxcore.h> // opencv
#include <cxerror.h> // opencv
#endif
#include "kernel/jafarException.hpp"

namespace jafar {

  namespace image {

    /** Base class for all exceptions defined in the module
     * image.
     *
     * @ingroup image
     */
    class ImageException : public ::jafar::kernel::Exception {

    public:

      /** This enumeration defines exceptions id for the module
       * image.
       */
      enum ExceptionId {
	INVALID_COLORSPACE_CONVERSION, /**< invalid color space convertion  */
	OPENCV_ERROR,
	NOT_IMPLEMENTED
      };

      /** Constructor. You should not use this constructor directly,
       * prefer macros jfrThrowEx or jfrCreateEx which fill for you
       * parameters \c file_ and \c line_.
       *
       * @param id_ exception id
       * @param message_ message used for debug
       * @param file_ where the exception was thrown
       * @param line_ where the exception was thrown
       */
      ImageException(ExceptionId id_, 
                            const std::string& message_, 
                            const std::string& file_, int line_) throw();

      virtual ~ImageException() throw();

      ExceptionId getExceptionId() const throw(); 

	/** Redirect OpenCV Error to Jafar Exception */
	static void enableOpenC_VException() {
		cvSetErrMode(CV_ErrModeParent);
		cvRedirectError(&SendErrorToJafar, 0, 0);
	};
	
    protected:

      ExceptionId id;

      static std::string exceptionIdToString(ExceptionId id_) throw();
      
	  static int SendErrorToJafar (int status, 
	    					 const char* func_name, 
	    					 const char* err_msg,
	    					 const char* file_name, 
	    					 int line, 
	    					 void* /*userdata*/ )
	    {
	    std::stringstream message;
	    message   
	        << " openCV Error:"
	        << "\n        Status=" << cvErrorStr(status)
	        << "\n        function name=" << (func_name ? func_name : "unknown")
	        << "\n        error message=" << (err_msg ? err_msg : "unknown")
	        << "\n        file_name=" << (file_name ? file_name : "unknown")
	        << "\n        line=" << line
	        << "\n        trace=" << kernel::get_system_trace()
	        << std::flush;

		jafar::kernel::print_system_trace();
	    // Clear OpenCV's error status for the next call!
	    cvSetErrStatus( CV_StsOk );

	    // Set Jafar Error.   
	    JFR_ERROR(ImageException,ImageException::OPENCV_ERROR,message.str());   
	    return 0;
	    }

    }; // class ImageException

  } // namespace image
} // namespace jafar

#endif // IMAGE_IMAGE_EXCEPTION_HPP

